@@ -18,45 +18,50 @@
 
 #include <config-amarok.h>
 
-#include "core/support/Amarok.h"
-#include "amarokconfig.h"
-#include "amarokurls/AmarokUrl.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Components.h"
-#include "core/interfaces/Logger.h"
 #include "ConfigDialog.h"
+#include "covermanager/CoverCache.h"
 #include "covermanager/CoverFetcher.h"
 #include "dialogs/EqualizerDialog.h"
 #include "dbus/CollectionDBusHandler.h"
 #include "core/support/Debug.h"
 #include "EngineController.h"
-#include "firstruntutorial/FirstRunTutorial.h"
 #include "KNotificationBackend.h"
-#include "core/capabilities/SourceInfoCapability.h"
-#include "core/meta/support/MetaConstants.h"
-#include "core/meta/Meta.h"
-#include "core/meta/support/MetaUtility.h"
-#include "core/transcoding/TranscodingController.h"
-#include "network/NetworkAccessManagerProxy.h"
 #include "Osd.h"
 #include "PlaybackConfig.h"
-#include "dbus/mpris1/PlayerHandler.h"
+#include "PluginManager.h"
+#include "ScriptManager.h"
+#include "TrayIcon.h"
+#include "amarokconfig.h"
+#include "amarokurls/AmarokUrl.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
+#include "core/capabilities/SourceInfoCapability.h"
+#include "core/interfaces/Logger.h"
+#include "core/meta/Meta.h"
+#include "core/meta/support/MetaConstants.h"
+#include "core/meta/support/MetaUtility.h"
 #include "core/playlists/Playlist.h"
 #include "core/playlists/PlaylistFormat.h"
-#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
-#include "playlist/PlaylistActions.h"
-#include "playlist/PlaylistModelStack.h"
-#include "playlist/PlaylistController.h"
-#include "playlistmanager/PlaylistManager.h"
-#include "core/plugins/PluginManager.h"
 #include "core/podcasts/PodcastProvider.h"
+#include "core/support/Amarok.h"
+#include "core/support/Components.h"
+#include "core/support/Debug.h"
+#include "core/transcoding/TranscodingController.h"
+#include "covermanager/CoverFetcher.h"
+#include "dbus/CollectionDBusHandler.h"
+#include "dbus/mpris1/PlayerHandler.h"
 #include "dbus/mpris1/RootHandler.h"
-#include "ScriptManager.h"
-#include "statemanagement/ApplicationController.h"
-#include "statemanagement/DefaultApplicationController.h"
 #include "dbus/mpris1/TrackListHandler.h"
 #include "dbus/mpris2/Mpris2DBusHandler.h"
-#include "TrayIcon.h"
+#include "dialogs/EqualizerDialog.h"
+#include "firstruntutorial/FirstRunTutorial.h"
+#include "network/NetworkAccessManagerProxy.h"
+#include "playlist/PlaylistActions.h"
+#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModelStack.h"
+#include "playlistmanager/PlaylistManager.h"
+#include "statemanagement/ApplicationController.h"
+#include "statemanagement/DefaultApplicationController.h"
 
 
 #ifdef NO_MYSQL_EMBEDDED
@@ -74,7 +79,7 @@
 #include <KIO/CopyJob>
 #include <KJob>
 #include <KJobUiDelegate>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KShortcutsDialog>              //slotConfigShortcuts()
 #include <KStandardDirs>
@@ -107,14 +112,6 @@ extern void setupEventHandler_mac(long);
 #endif // DEBUG
 
 QStringList App::s_delayedAmarokUrls = QStringList();
-
-AMAROK_EXPORT KAboutData aboutData( "amarok", 0,
-    ki18n( "Amarok" ), AMAROK_VERSION,
-    ki18n( "The audio player for KDE" ), KAboutData::License_GPL,
-    ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2010, The Amarok Development Squad" ),
-    ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: %1)" ).subs( __DATE__ ),
-             ( "http://amarok.kde.org" ) );
-
 AMAROK_EXPORT OcsData ocsData( "opendesktop" );
 
 App::App()
@@ -152,6 +149,8 @@ App::App()
     qRegisterMetaType<Meta::YearList>();
     qRegisterMetaType<Meta::LabelPtr>();
     qRegisterMetaType<Meta::LabelList>();
+    qRegisterMetaType<Playlists::PlaylistPtr>();
+    qRegisterMetaType<Playlists::PlaylistList>();
 
 
     //make sure we have enough cache space for all our crazy svg stuff
@@ -266,9 +265,11 @@ App::~App()
     Playlist::ModelStack::destroy();
     Playlist::Actions::destroy();
     PlaylistManager::destroy();
-    CollectionManager::destroy();
     CoverFetcher::destroy();
+    CoverCache::destroy();
+    CollectionManager::destroy();
     NetworkAccessManagerProxy::destroy();
+    Plugins::PluginManager::destroy();
 
     //this should be moved to App::quit() I guess
     Amarok::Components::applicationController()->shutdown();
@@ -422,14 +423,6 @@ App::handleCliArgs() //static
 // INIT
 /////////////////////////////////////////////////////////////////////////////////////
 
-void
-App::initCliArgs( int argc, char *argv[] )
-{
-    KCmdLineArgs::reset();
-    KCmdLineArgs::init( argc, argv, &::aboutData ); //calls KCmdLineArgs::addStdCmdLineOptions()
-    initCliArgs();
-}
-
 void
 App::initCliArgs() //static
 {
@@ -608,7 +601,7 @@ App::continueInit()
     Amarok::Components::applicationController()->start();
 
     KSplashScreen* splash = 0;
-    if( AmarokConfig::showSplashscreen() && !isSessionRestored() )
+    if( AmarokConfig::showSplashScreen() && !isSessionRestored() )
     {
         QPixmap splashimg( KGlobal::dirs()->findResource( "data", "amarok/images/splash_screen.jpg" ) );
         splash = new KSplashScreen( splashimg, Qt::WindowStaysOnTopHint );
@@ -668,7 +661,7 @@ App::continueInit()
 
     //Instantiate the Transcoding::Controller, this fires up an asynchronous KProcess with
     //FFmpeg which should not take more than ~200msec.
-    The::transcodingController();
+    Amarok::Components::setTranscodingController( new Transcoding::Controller( this ) );
 
     if( splash ) // close splash correctly
     {
@@ -735,17 +728,14 @@ App::continueInit()
 
 void App::slotConfigAmarok( const QString& page )
 {
-    Amarok2ConfigDialog* dialog = static_cast<Amarok2ConfigDialog*>( KConfigDialog::exists( "settings" ) );
-
+    KConfigDialog *dialog = KConfigDialog::exists( "settings" );
     if( !dialog )
     {
         //KConfigDialog didn't find an instance of this dialog, so lets create it :
         dialog = new Amarok2ConfigDialog( mainWindow(), "settings", AmarokConfig::self() );
-
-        connect( dialog, SIGNAL( settingsChanged( const QString& ) ), SLOT( applySettings() ) );
+        connect( dialog, SIGNAL(settingsChanged(QString)), SLOT(applySettings()) );
     }
-
-    dialog->show( page );
+    static_cast<Amarok2ConfigDialog*>( dialog )->show( page );
 }
 
 void App::slotConfigShortcuts()
