@@ -88,7 +88,10 @@ DirectoryLoader::init( const QList<KUrl>& urls )
             {
                 Playlists::PlaylistFilePtr playlist = Playlists::loadPlaylistFile( kurl );
                 if( playlist )
+                {
+                    playlist->triggerTrackLoad(); //playlist track loading is on demand.
                     m_tracks << playlist->tracks();
+                }
             }
             else
             {
