@@ -61,10 +61,10 @@ public:
         return merged;
     }
 
-    // %something gets replaced by the value corresponding to key "something" in args
+    // %something% gets replaced by the value corresponding to key "something" in args
     QString namedArgs( const QMap<QString, QString> &args, bool opt=false ) const
     {
-        QRegExp rxArg( "%[a-zA-Z0-9]+" );
+        QRegExp rxArg( "%[a-zA-Z0-9]+%" );
 
         QString result;
         int start = 0;
@@ -73,7 +73,7 @@ public:
                 pos = rxArg.indexIn( *this, start ) )
         {
             int len = rxArg.matchedLength();
-            QString p = rxArg.capturedTexts()[0].mid(1, len-1);
+            QString p = rxArg.capturedTexts()[0].mid(1, len - 2);
 
             result += mid( start, pos-start );
             if( !args[p].isEmpty() )
@@ -88,12 +88,12 @@ public:
         return result;
     }
 
-    // %something gets replaced by the value corresponding to key "something" in args,
+    // %something% gets replaced by the value corresponding to key "something" in args,
     // however, if key "something" is not available,
     // then replace everything within surrounding { } by an empty string
     QString namedOptArgs( const QMap<QString, QString> &args ) const
     {
-        QRegExp rxOptArg( "%[a-zA-Z0-9]+" );
+        QRegExp rxOptArg( "%[a-zA-Z0-9]+%" );
 
         QString result = *this;
         for( int pos = rxOptArg.indexIn( result );
@@ -136,17 +136,19 @@ public:
             if( bracketCount == 0 ) // syntax seems to be correct
                 rightMatchPos = i;
 
-            if( !args.contains( rxOptArg.capturedTexts()[0].mid( 1, len ) ) ) // remove section
+            if( !args.contains( rxOptArg.capturedTexts()[0].mid( 1, len - 2 ) ) ) // remove section
                 result.remove( leftMatchPos, rightMatchPos - leftMatchPos + 1 );
             else // we have a map entry
             {
                 if( result.at( rightMatchPos ) == '}' )
                     result.remove( rightMatchPos, 1 );
-
-                result.replace( pos, len, args[ rxOptArg.capturedTexts()[0].mid( 1, len-1 ) ] );
-
                 if( result.at( leftMatchPos ) == '{' )
+                {
                     result.remove( leftMatchPos, 1 );
+                    pos--;
+                }
+
+                result.replace( pos, len, args[ rxOptArg.capturedTexts()[0].mid( 1, len - 2 ) ] );
             }
         }
         return result;
