@@ -32,14 +32,17 @@
 #include "widgets/ContainmentArrow.h"
 #include "widgets/appletexplorer/AppletExplorer.h"
 
-#include <plasma/containment.h>
-#include <plasma/view.h>
-#include <QMouseEvent>
+#include <Plasma/Containment>
+#include <Plasma/View>
 
+#include <QMouseEvent>
 #include <QGraphicsView>
+#include <QQueue>
+#include <QWeakPointer>
 
 class QPixmap;
 class ContextUrlRunner;
+class QParallelAnimationGroup;
 
 namespace Context
 {
@@ -47,7 +50,7 @@ namespace Context
 class ContextScene;
 class ControlBox;
 
-class AMAROK_EXPORT ContextView : public Plasma::View, public Engine::EngineObserver, public ContextSubject
+class AMAROK_EXPORT ContextView : public Plasma::View, public ContextSubject
 {
     Q_OBJECT
 
@@ -72,8 +75,8 @@ public:
     void clear( const ContextState& name );
 
     void clearNoSave();
-    
-    /** 
+
+    /**
         Shows the home state. Loads applets from config file.
     */
     void showHome();
@@ -88,9 +91,9 @@ public:
     */
     QStringList currentAppletNames();
 
+    void addCollapseAnimation( QAbstractAnimation *anim );
 
 public slots:
-
     /**
      * Add the applet with the given plugin name to the context view. Will add in default position, which is at
      *  the end of the applet list.
@@ -104,42 +107,32 @@ public slots:
 
 signals:
     void appletExplorerHid();
-        
+
 protected:
-    
-    virtual void enginePlaybackEnded( qint64 finalPosition, qint64 trackLength, PlaybackEndedReason reason );
-    void engineNewMetaData( const QHash<qint64, QString> &newMetaData, bool trackChanged ); // for stream scrobbling
-    virtual void engineNewTrackPlaying();
-    
     void resizeEvent(QResizeEvent *event);
     void wheelEvent(QWheelEvent *event);
 
+private slots:
+    void slotTrackChanged( Meta::TrackPtr track );
+    void slotMetadataChanged( Meta::TrackPtr track );
+    void slotPositionAppletExplorer();
+    void slotStartCollapseAnimations();
+    void slotCollapseAnimationsFinished();
+
 private:
     static ContextView* s_self;
 
-    /**
-    * Set all containments geometry in the scene with the same geometry as the Context View widget 
-    */
-    void updateContainmentsGeometry();
-      
     void loadConfig();
 
-    typedef QPointer< Context::Applet > AppletPointer;    
-
     // holds what is currently being shown
     ContextState m_curState;
 
-    //it seems we get a Phonon::PausedState before we start to play anything.
-    //Because we generally don't want to update the context view when moving from
-    //Paused to Playing state, this causes the CV to not get updated when starting Amarok
-    //with a track being resumed (Resume playback enabled in the options). To avoid this,
-     //we always kick the cv on the first play state we receive, irregardless if the
-     //previous state was Paused.
-    bool m_firstPlayingState;
-
     ContextUrlRunner * m_urlRunner;
 
-    AppletExplorer* m_appletExplorer;
+    QWeakPointer<AppletExplorer> m_appletExplorer;
+    QWeakPointer<QParallelAnimationGroup> m_collapseAnimations;
+    QQueue< QWeakPointer<QAbstractAnimation> > m_queuedAnimations;
+    QWeakPointer<QTimer> m_collapseGroupTimer;
 };
 
 } // Context namespace
