@@ -19,13 +19,14 @@
 
 #include "PopupDropper.h"
 #include "PopupDropper_p.h"
-#include "PopupDropperAction.h"
 #include "PopupDropperItem.h"
 
 #include <QtDebug>
 #include <QGraphicsItem>
 #include <QGraphicsScene>
 #include <QtSvg/QSvgRenderer>
+#include <QAction>
+#include <QMenu>
 #include <QPalette>
 #include <QTimeLine>
 #include <QWidget>
@@ -115,6 +116,8 @@ void PopupDropperPrivate::fadeHideTimerFrameChanged( int frame ) //SLOT
         int alpha = (int)( color.alpha() * val );
         color.setAlpha( alpha );
         q->setPalette( color );
+        foreach( PopupDropperItem* pdi, pdiItems )
+            pdi->setSubitemOpacity( val );
     }
 }
 
@@ -127,19 +130,24 @@ void PopupDropperPrivate::fadeShowTimerFrameChanged( int frame ) //SLOT
         int alpha = (int)( color.alpha() * val );
         color.setAlpha( alpha );
         q->setPalette( color );
+        foreach( PopupDropperItem* pdi, pdiItems )
+            pdi->setSubitemOpacity( val );
     }
 }
 
 void PopupDropperPrivate::fadeHideTimerFinished() //SLOT
 {
     view->hide();
-    //qDebug() << "Emitting fadeHideFinished";
+    //qDebug() << "Emitting fadeHideFinished in d pointer " << this;
     emit q->fadeHideFinished();
 }
 
 void PopupDropperPrivate::fadeShowTimerFinished() //SLOT
 {
     q->setPalette( windowColor ); 
+    queuedHide = false;
+    foreach( PopupDropperItem* pdi, pdiItems )
+        pdi->setSubitemOpacity( 1.0 );
 }
 
 void PopupDropperPrivate::dragEntered()
@@ -261,6 +269,7 @@ int PopupDropper::overlayLevel() const
 void PopupDropper::initOverlay( QWidget* parent, PopupDropperPrivate* priv )
 {
     PopupDropperPrivate *pdp = priv ? priv : d;
+    //qDebug() << "PUD Overlay being created, d pointer is " << d;
     pdp->scene->setSceneRect( QRectF( parent->rect() ) );
     //qDebug() << "Scene width = " << pdp->scene->width();
     pdp->scene->setItemIndexMethod( QGraphicsScene::NoIndex );
@@ -281,8 +290,8 @@ void PopupDropper::addOverlay()
     m_viewStack.push( d );
     PopupDropperPrivate* old_d = d;
     d = new PopupDropperPrivate( this, false, old_d->view );
-    d->onTop = true;
     d->sharedRenderer = old_d->sharedRenderer;
+    //qDebug() << "Adding overlay: ";
     initOverlay( old_d->view );
     setColors( d->windowColor, d->baseTextColor, d->hoveredTextColor, d->hoveredBorderPen.color(), d->hoveredFillBrush.color() );
     d->quitOnDragLeave = true;
@@ -311,10 +320,11 @@ void PopupDropper::addOverlay( PopupDropperPrivate* newD )
 //with hide() (which will hide just one level)
 bool PopupDropper::subtractOverlay()
 {
-    //qDebug() << "subtractOverlay";
-    disconnect( this, SIGNAL( fadeHideFinished() ), this, SLOT( subtractOverlay() ) );
+    //qDebug() << "subtractOverlay, with d pointer " << d;
+    disconnect( this, SLOT( subtractOverlay() ) );
     while( !isHidden() && d->fadeHideTimer.state() == QTimeLine::Running )
     {
+        //qDebug() << "singleshotting subtractOverlay";
         QTimer::singleShot( 0, this, SLOT( subtractOverlay() ) );
         return false;
     }
@@ -347,11 +357,6 @@ bool PopupDropper::subtractOverlay()
 }
 
 PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, const QString &text )
-{
-    return addSubmenu( pd, 0, QString(), text );
-}
-
-PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, QSvgRenderer *renderer, const QString &elementId, const QString &text )
 {
     //qDebug() << "addSubmenu, this is " << this << " and passed-in PopupDropper is " << (*pd);
     if( !(*pd) )
@@ -369,20 +374,17 @@ PopupDropperItem* PopupDropper::addSubmenu( PopupDropper** pd, QSvgRenderer *ren
     initOverlay( d->widget, newD );
     PopupDropperItem* pdi = new PopupDropperItem();
 
-    PopupDropperAction* pda = 0;
+    QAction* action = new QAction( text, this );
 
-    if( renderer )
-        pda = new PopupDropperAction( renderer, elementId, text, this );
-    else
-        pda = new PopupDropperAction( text, this );
-
-    connect( pda, SIGNAL( hovered() ), this, SLOT( activateSubmenu() ) );
-    pdi->setAction( pda );
+    connect( action, SIGNAL( hovered() ), this, SLOT( activateSubmenu() ) );
+    pdi->setAction( action );
     pdi->setSubmenuTrigger( true );
     pdi->setHoverIndicatorShowStyle( PopupDropperItem::OnHover );
-    d->submenuMap[pda] = newD;
+    d->submenuMap[action] = newD;
     delete (*pd);
     (*pd) = 0;
+    foreach( PopupDropperItem* item, newD->pdiItems )
+        item->setPopupDropper( this );
     //qDebug() << "d->submenuMap[pda] = " << d->submenuMap[pda];
     addItem( pdi );
     return pdi;
@@ -394,7 +396,7 @@ void PopupDropper::activateSubmenu()
     if( isHidden() || d->fadeHideTimer.state() == QTimeLine::Running )
         return;
     PopupDropperPrivate* oldd = d;
-    addOverlay( d->submenuMap[static_cast<PopupDropperAction*>(QObject::sender())] );
+    addOverlay( d->submenuMap[static_cast<QAction*>(QObject::sender())] );
     //qDebug() << "d->pdiItems.size() = " << d->pdiItems.size() << " for " << d;
     foreach( PopupDropperItem* item, d->pdiItems )
         addItem( item, false, false );
@@ -402,6 +404,37 @@ void PopupDropper::activateSubmenu()
     show();
 }
 
+bool PopupDropper::addMenu( const QMenu *menu )
+{
+    Q_UNUSED(menu)
+    if( !menu )
+        return false;
+        
+    if( menu->actions().size() == 0 )
+        return true;
+
+    PopupDropperItem *pdi = 0;
+    foreach( QAction *action, menu->actions() )
+    {
+        if( !action->menu() )
+        {
+            pdi = new PopupDropperItem();
+            pdi->setAction( action );
+            addItem( pdi );
+        }
+        else
+        {
+            PopupDropper *pd = new PopupDropper( 0 );
+            bool success = pd->addMenu( action->menu() );
+            if( success )
+                pdi = addSubmenu( &pd, action->text() );
+        }
+        pdi = 0;
+    }
+    
+    return true;
+}
+
 bool PopupDropper::standalone() const
 {
     return d->standalone;
@@ -432,6 +465,8 @@ void PopupDropper::show()
         QColor color = d->windowColor;
         color.setAlpha( 0 );
         setPalette( color );
+        foreach( PopupDropperItem* pdi, d->pdiItems )
+            pdi->setSubitemOpacity( 0.0 );
         d->fadeShowTimer.start();
         //qDebug() << "Timer started";
     }
@@ -452,46 +487,47 @@ void PopupDropper::showAllOverlays()
 //returns immediately! 
 void PopupDropper::hide()
 {
-    //qDebug() << "PopupDropper::hide entered, isHidden = " << (isHidden() ? "true":"false");
+    //qDebug() << "PopupDropper::hide entered, d pointer is = " << d;
+
+    //if hide is called and the view is already hidden, it's likely spurious
     if( isHidden() )
     {
-        emit fadeHideFinished();
+        //qDebug() << "ishidden, returning";
         return;
     }
 
+    //queuedHide is to make sure that fadeShowTimerFinished executes before this next hide()
     if( d->fadeShowTimer.state() == QTimeLine::Running )
     {
-        if( !d->queuedHide )
-        {
-            QTimer::singleShot( 0, this, SLOT( hide() ) );
-            d->queuedHide = true;
-        }
+        //qDebug() << "show timer running, queueing hide";
+        d->fadeShowTimer.stop();
+        d->queuedHide = true;
+        QTimer::singleShot( 0, d, SLOT( fadeShowTimerFinished() ) );
+        QTimer::singleShot( 0, this, SLOT( hide() ) );
         return;
     }
 
-    if( d->fadeHideTimer.state() == QTimeLine::Running )
-        return;
-
-    bool wasRunning = false;
-    if( d->fadeHideTimer.state() == QTimeLine::Running )
+    //queuedHide will be set to false from fadeShowTimerFinished...so if this came up first,
+    //then wait
+    if( d->fadeHideTimer.state() == QTimeLine::Running || d->queuedHide )
     {
-        d->fadeHideTimer.stop();
-        wasRunning = true;
+        //qDebug() << "hide timer running or queued hide";
+        QTimer::singleShot( 0, this, SLOT( hide() ) );
+        return;
     }
+
     if( ( d->fade == PopupDropper::FadeOut || d->fade == PopupDropper::FadeInOut ) && d->fadeOutTime > 0 )
     {
+        //qDebug() << "Starting fade out";
         d->fadeHideTimer.setDuration( d->fadeOutTime );
-        if( !wasRunning )
-            d->fadeHideTimer.setCurrentTime( d->fadeOutTime );
         d->fadeHideTimer.setCurveShape( QTimeLine::LinearCurve );
         d->fadeHideTimer.start();
         //qDebug() << "Timer started";
         return;
     }
-    if( !d->queuedHide )
+    else  //time is zero, or no fade
     {
-        //make sure weird issues with the timeline don't cause this to trigger twice
-        d->queuedHide = true;
+        //qDebug() << "time is zero, or no fade";
         QTimer::singleShot( 0, d, SLOT( fadeHideTimerFinished() ) );
         return;
     }
@@ -499,25 +535,25 @@ void PopupDropper::hide()
 
 void PopupDropper::hideAllOverlays()
 {
-    qDebug() << "Entered hideAllOverlays";
+    //qDebug() << "Entered hideAllOverlays";
     connect( this, SIGNAL( fadeHideFinished() ), this, SLOT( slotHideAllOverlays() ) );
     hide();
-    qDebug() << "Leaving hideAllOverlays";
+    //qDebug() << "Leaving hideAllOverlays";
 }
 
 void PopupDropper::slotHideAllOverlays()
 {
-    qDebug() << "Entered slotHideAllOverlays()";
+    //qDebug() << "Entered slotHideAllOverlays()";
     disconnect( this, SIGNAL( fadeHideFinished() ), this, SLOT( slotHideAllOverlays() ) );
-    qDebug() << "m_viewStack.size() = " << m_viewStack.size();
+    //qDebug() << "m_viewStack.size() = " << m_viewStack.size();
     for( int i = m_viewStack.size() - 1; i >= 0; --i )
     {
         PopupDropperPrivate* pdp = m_viewStack.at( i );
-        qDebug() << "checking pdp = " << (QObject*)pdp << ", d is " << (QObject*)d;
+        //qDebug() << "checking pdp = " << (QObject*)pdp << ", d is " << (QObject*)d;
         if( pdp != d )
             pdp->view->hide();
     }
-    qDebug() << "Leaving slotHideAllOverlays";
+    //qDebug() << "Leaving slotHideAllOverlays";
 }
 
 void PopupDropper::update()
@@ -549,15 +585,14 @@ void PopupDropper::clear()
         return;
     }
     //qDebug() << "Clear happening!";
-    disconnect( &d->fadeHideTimer, SIGNAL( finished() ), this, SLOT( clear() ) );
-    disconnect( &d->fadeShowTimer, SIGNAL( finished() ), this, SLOT( clear() ) );
+    disconnect( this, SLOT( clear() ) );
     do
     {
         foreach( QGraphicsItem* item, d->allItems )
         {
             if( dynamic_cast<PopupDropperItem*>(item) )
             {
-                if( dynamic_cast<PopupDropperItem*>(item)->submenuTrigger() )
+                if( dynamic_cast<PopupDropperItem*>(item)->isSubmenuTrigger() )
                 {
                     //qDebug() << "Disconnecting action";
                     disconnect( dynamic_cast<PopupDropperItem*>(item)->action(), SIGNAL( hovered() ), this, SLOT( activateSubmenu() ) );
@@ -821,9 +856,9 @@ void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer )
 void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer, bool appendToList )
 {
     //qDebug() << "adding item";
-    //FIXME: Make separators use something graphical instead of just ignoring them
+    //FIXME: Make separators do something graphical instead of just ignoring them
     PopupDropperItem *pItem = static_cast<PopupDropperItem*>( item );
-    if( pItem->action()->isSeparator() )
+    if( pItem->isSeparator() )
         return;
     if( useSharedRenderer )
         pItem->setSharedRenderer( d->sharedRenderer );
@@ -867,20 +902,60 @@ void PopupDropper::addItem( PopupDropperItem *item, bool useSharedRenderer, bool
     d->scene->addItem( pItem );
 }
 
-void PopupDropper::addSeparator( PopupDropperAction* separator )
+QList<PopupDropperItem*> PopupDropper::items() const
 {
-    if( separator && !separator->isSeparator() )
+    QList<PopupDropperItem*> list;
+    foreach( PopupDropperItem *item, d->pdiItems )
+        list.append( item );
+
+    return list;
+}
+
+//Won't currently work for > 1 level of submenu!
+//TODO: Figure out a better way. (Does anything else work > 1 level?)
+QList<PopupDropperItem*> PopupDropper::submenuItems( const PopupDropperItem *item ) const
+{
+    QList<PopupDropperItem*> list;
+    if( !item || !item->isSubmenuTrigger() || !d->submenuMap.contains( item->action() ) )
+        return list;
+
+    PopupDropperPrivate *pdp = d->submenuMap[item->action()];
+    foreach( PopupDropperItem *pdi, pdp->pdiItems )
+        list.append( pdi );
+
+    return list;
+}
+
+//Goes through and calls the callback on all items, including submenuItems
+//which can be adjusted differently by checking isSubmenuTrigger()
+void PopupDropper::forEachItem( void callback(void*) )
+{
+    forEachItemPrivate( d, callback );
+}
+
+void PopupDropper::forEachItemPrivate( PopupDropperPrivate *pdp, void callback(void* item) )
+{
+    foreach( PopupDropperItem *item, pdp->pdiItems )
+        callback( item );
+    foreach( QAction *action, pdp->submenuMap.keys() )
+        forEachItemPrivate( pdp->submenuMap[action], callback );
+}
+
+void PopupDropper::addSeparator( PopupDropperItem* separator )
+{
+
+    if( !separator )
     {
         //qDebug() << "Action is not a separator!";
         return;
     }
 
-    if( separator && separator->separatorStyle() == PopupDropperAction::TextSeparator )
+    separator->setSeparator( true );
+
+    if( separator->separatorStyle() == PopupDropperItem::TextSeparator )
     {
         //qDebug() << "Separator style is text";
-        PopupDropperItem* pdi = new PopupDropperItem();
-        pdi->setAction( separator );
-        addItem( pdi );
+        addItem( separator );
     }
 
     //qDebug() << "Separator style is line";
