@@ -783,13 +783,13 @@ DatabaseUpdater::createTables() const
     {
         QString c = "CREATE TABLE statistics "
                     "(id " + storage->idType() +
-                    ",url INTEGER"
+                    ",url INTEGER NOT NULL"
                     ",createdate INTEGER" // this is the first played time
                     ",accessdate INTEGER" // this is the last played time
                     ",score FLOAT"
-                    ",rating INTEGER DEFAULT 0"
-                    ",playcount INTEGER"
-                    ",deleted BOOL DEFAULT " + storage->boolFalse() +
+                    ",rating INTEGER NOT NULL DEFAULT 0" // the "default" undefined rating is 0. We cannot display anything else.
+                    ",playcount INTEGER NOT NULL DEFAULT 0" // a track is either played or not.
+                    ",deleted BOOL NOT NULL DEFAULT " + storage->boolFalse() +
                     ") COLLATE = utf8_bin ENGINE = MyISAM;";
         storage->query( c );
         storage->query( "CREATE UNIQUE INDEX statistics_url ON statistics(url);" );
@@ -866,9 +866,19 @@ DatabaseUpdater::adminValue( const QString &key ) const
 {
     SqlStorage *storage = m_collection->sqlStorage();
 
-    QStringList values;
-    values = storage->query( QString( "SELECT version FROM admin WHERE component = '%1';").arg(storage->escape( key ) ) );
-    return values.isEmpty() ? 0 : values.first().toInt();
+    QStringList columns = storage->query(
+            QString( "SELECT column_name FROM INFORMATION_SCHEMA.columns "
+                     "WHERE table_name='admin'" ) );
+    if( columns.isEmpty() )
+        return 0; //no table with that name
+
+    QStringList values = storage->query(
+            QString( "SELECT version FROM admin WHERE component = '%1';")
+            .arg(storage->escape( key ) ) );
+    if( values.isEmpty() )
+        return 0;
+
+    return values.first().toInt();
 }
 
 void
