@@ -25,14 +25,20 @@
 #include "core/support/Debug.h"
 #include "core/transcoding/TranscodingController.h"
 #include "core-impl/collections/db/ScanManager.h"
-#include "core-impl/collections/db/sql/MountPointManager.h"
+#include "MountPointManager.h"
 #include "SqlCollectionLocation.h"
 #include "SqlQueryMaker.h"
 #include "SqlScanResultProcessor.h"
 #include "SvgHandler.h"
 
+#include "MainWindow.h"
+#include <KMessageBox>
+
 /*
  * #ifdef Q_OS_WIN32
+ *
+ * #include <QTimer>
+ *
  * class XesamCollectionBuilder
  * {
  * public:
@@ -43,14 +49,7 @@
  * #endif
  */
 
-#include <klocale.h>
-#include <KIcon>
-#include <KMessageBox> // TODO put the delete confirmation code somewhere else?
-
 #include "dialogs/OrganizeCollectionDialog.h"
-#include "MainWindow.h"
-
-#include <QTimer>
 
 namespace Collections {
 
@@ -69,7 +68,7 @@ public:
     virtual void setTranscodingConfiguration( const Transcoding::Configuration &configuration
                                                   = Transcoding::Configuration() )
     { m_targetFileExtension =
-      The::transcodingController()->format( configuration.encoder() )->fileExtension(); }
+      Amarok::Components::transcodingController()->format( configuration.encoder() )->fileExtension(); }
 
     virtual void show()
     {
@@ -165,17 +164,29 @@ SqlCollection::SqlCollection( const QString &id, const QString &prettyName, SqlS
     m_collectionLocationFactory = new SqlCollectionLocationFactoryImpl( this );
     m_queryMakerFactory = new DefaultSqlQueryMakerFactory( this );
     m_scanManager = new ScanManager( this, this );
+
+    // we need a UI-dialog service, but for now just output the messages
+    if( !storage->getLastErrors().isEmpty() )
+    {
+        KMessageBox::error( The::mainWindow(), //parent
+                            i18n( "The amarok database reported the following errors:\n"
+                                  "%1\n"
+                                  "In most cases you will need to resolve these errors before Amarok will run properly." ).
+                            arg(storage->getLastErrors().join("\n")) );
+    }
 }
 
 SqlCollection::~SqlCollection()
 {
-    delete m_registry;
+    if( m_scanManager )
+        m_scanManager->abort( "Amarok is closing" );
     delete m_albumCapabilityDelegate;
     delete m_artistCapabilityDelegate;
     delete m_trackCapabilityDelegate;
     delete m_collectionLocationFactory;
     delete m_queryMakerFactory;
     delete m_sqlStorage;
+    delete m_registry;
     delete m_mpm;
 }
 
