@@ -29,11 +29,12 @@
 #include "core/meta/Meta.h"
 #include "core/meta/support/MetaUtility.h"
 #include "core-impl/collections/db/ScanManager.h"
-#include "core-impl/collections/db/sql/MountPointManager.h"
+#include "MountPointManager.h"
 #include "SqlCollection.h"
 #include "SqlMeta.h"
 #include "transcoding/TranscodingJob.h"
 #include "core/transcoding/TranscodingController.h"
+#include <shared/MetaTagLib.h> // for getting the uid
 
 #include <QDir>
 #include <QFile>
@@ -171,9 +172,13 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     QString rpath = m_collection->mountPointManager()->getRelativePath( deviceId, url );
     int directoryId = registry->getDirectory( QFileInfo(url).path() );
 
+    // -- the track uid (we can't use the original one from the old collection)
+    Meta::FieldHash fileTags = Meta::Tag::readTags( url );
+    QString uid = fileTags.value( Meta::valUniqueId ).toString();
+
     // -- the the track from the registry
     KSharedPtr<Meta::SqlTrack> metaTrack;
-    metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrackFromUid( track->uidUrl() ) );
+    metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrackFromUid( uid ) );
 
     if( metaTrack ) {
         warning() << "Location is inserting a file with the same uid as an already existing one.";
@@ -181,7 +186,7 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
         metaTrack->setUrl( deviceId, rpath, directoryId );
 
     } else {
-        metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrack( deviceId, rpath, directoryId, track->uidUrl() ) );
+        metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrack( deviceId, rpath, directoryId, uid ) );
     }
 
     // -- set the values
@@ -192,12 +197,7 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
         metaTrack->setTitle( track->name() );
 
     if( track->album() )
-    {
         metaTrack->setAlbum( track->album()->name() );
-        // also copy over the album cover
-        if( track->album()->hasImage() && !metaTrack->album()->hasImage() )
-            metaTrack->album()->setImage( track->album()->image().toImage() );
-    }
 
     if( track->artist() )
         metaTrack->setArtist( track->artist()->name() );
@@ -205,7 +205,7 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     if( track->composer() )
         metaTrack->setComposer( track->composer()->name() );
 
-    if( !track->year() >= 0 )
+    if( track->year() > 0 )
         metaTrack->setYear( track->year() );
 
     if( track->genre() )
@@ -217,10 +217,11 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     /* we've already done this
     if( !track->path().isEmpty() )
         metaTrack->setUrl( track->path() );
-        */
 
+     and that too
     if( !track->uidUrl().isEmpty() )
-        metaTrack->setUidUrl( track->uidUrl() );
+        metaTrack->setUidUrl( uid );
+    */
 
     if( !track->bpm() >= 0 )
         metaTrack->setBpm( track->bpm() );
@@ -260,8 +261,19 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
         metaTrack->addLabel( label );
 
     metaTrack->endMetaDataUpdate();
+
+    // Used to be updated after changes commit to prevent crash on NULL pointer access
+    // if metaTrack had no album.
+    if( track->album() && track->album()->hasImage() && !metaTrack->album()->hasImage() )
+        metaTrack->album()->setImage( track->album()->image() );
+
     metaTrack->setWriteFile( true );
 
+    // we have a first shot at the meta data (expecially ratings and playcounts from media
+    // collections) but we still need to trigger the collection scanner
+    // to get the album and other meta data correct.
+    m_collection->scanManager()->delayedIncrementalScan( QFileInfo(url).path() );
+
     return true;
 }
 
@@ -364,13 +376,17 @@ void
 SqlCollectionLocation::slotJobFinished( KJob *job )
 {
     DEBUG_BLOCK
+
+    Meta::TrackPtr track = m_jobs.value( job );
     if( job->error() )
     {
         //TODO: proper error handling
         warning() << "An error occurred when copying a file: " << job->errorString();
-        source()->transferError(m_jobs.value( job ), KIO::buildErrorString( job->error(), job->errorString() ) );
-        m_destinations.remove( m_jobs.value( job ) );
+        source()->transferError( track, KIO::buildErrorString( job->error(), job->errorString() ) );
+        m_destinations.remove( track );
     }
+    else
+        source()->transferSuccessful( track );
 
     m_jobs.remove( job );
     job->deleteLater();
@@ -381,21 +397,25 @@ void
 SqlCollectionLocation::slotRemoveJobFinished( KJob *job )
 {
     DEBUG_BLOCK
-    int error = job->error();
-    if( error != 0 && error != KIO::ERR_DOES_NOT_EXIST )
+    Meta::TrackPtr track = m_removejobs.value( job );
+    if( job->error() )
     {
         //TODO: proper error handling
-        debug() << "KIO::ERR_DOES_NOT_EXIST" << KIO::ERR_DOES_NOT_EXIST;
         warning() << "An error occurred when removing a file: " << job->errorString();
-        transferError(m_removejobs.value( job ), KIO::buildErrorString( job->error(), job->errorString() ) );
+    }
+
+    // -- remove the track from the database if it's gone
+    if( !QFile(track->playableUrl().path()).exists() )
+    {
+        // Remove the track from the database
+        remove( track );
+
+        //we  assume that KIO works correctly...
+        transferSuccessful( track );
     }
     else
     {
-        // Remove the track from the database
-        remove( m_removejobs.value( job ) );
-
-        //we  assume that KIO works correctly...
-        transferSuccessful( m_removejobs.value( job ) );
+        transferError( track, KIO::buildErrorString( job->error(), job->errorString() ) );
     }
 
     m_removejobs.remove( job );
@@ -420,13 +440,9 @@ void SqlCollectionLocation::slotTransferJobFinished( KJob* job )
     // that were successfully copied
     foreach( const Meta::TrackPtr &track, m_destinations.keys() )
     {
-        if( !QFileInfo( m_destinations[ track ] ).exists() )
-            m_destinations.remove( track );
+        if( QFile::exists( m_destinations[ track ] ) )
+            insert( track, m_destinations[ track ] );
         m_originalUrls[track] = track->playableUrl();
-        debug() << "inserting original url" << m_originalUrls[track];
-        // report successful transfer with the original url
-        source()->transferSuccessful( track );
-        insert( track, m_destinations[ track ] );
     }
     debug () << "m_originalUrls" << m_originalUrls;
     m_collection->scanManager()->unblockScan();
@@ -443,9 +459,7 @@ void SqlCollectionLocation::slotTransferJobAborted()
     // that were successfully copied
     foreach( const Meta::TrackPtr &track, m_destinations.keys() )
     {
-        if( !QFileInfo( m_destinations[ track ] ).exists() )
-            m_destinations.remove( track );
-        else
+        if( QFile::exists( m_destinations[ track ] ) )
             insert( track, m_destinations[ track ] ); // was already copied, so have to insert it in the db
         m_originalUrls[track] = track->playableUrl();
     }
@@ -515,8 +529,10 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
 
         KUrl dest = m_destinations[ track ];
         dest.cleanPath();
-
         src.cleanPath();
+
+        bool hasMoodFile = QFile::exists( moodFile( src ).toLocalFile() );
+
         if( configuration.encoder() == Transcoding::NULL_CODEC )
             debug() << "copying from " << src << " to " << dest;
         else
@@ -545,11 +561,11 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         }
 
         KJob *job = 0;
+        KJob *moodJob = 0;
 
         if( src.equals( dest ) )
         {
-            debug() << "move to itself found";
-            source()->transferSuccessful( track );
+            warning() << "move to itself found: " << info.absoluteFilePath();
             m_transferjob->slotJobFinished( 0 );
             if( m_sources.isEmpty() )
                 return false;
@@ -559,6 +575,12 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         {
             debug() << "moving!";
             job = KIO::file_move( src, dest, -1, flags );
+            if( hasMoodFile )
+            {
+                KUrl moodSrc = moodFile( src );
+                KUrl moodDest = moodFile( dest );
+                moodJob = KIO::file_move( moodSrc, moodDest, -1, flags );
+            }
         }
         else
         {
@@ -569,18 +591,32 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
             {
                 QString destPath = dest.path();
                 destPath.truncate( dest.path().lastIndexOf( '.' ) + 1 );
-                destPath.append( The::transcodingController()->
+                destPath.append( Amarok::Components::transcodingController()->
                                  format( configuration.encoder() )->fileExtension() );
                 dest.setPath( destPath );
                 job = new Transcoding::Job( src, dest, configuration, this );
                 job->start();
             }
+
+            if( hasMoodFile )
+            {
+                KUrl moodSrc = moodFile( src );
+                KUrl moodDest = moodFile( dest );
+                moodJob = KIO::file_copy( moodSrc, moodDest, -1, flags );
+            }
         }
         if( job )   //just to be safe
         {
             connect( job, SIGNAL( result( KJob* ) ), SLOT( slotJobFinished( KJob* ) ) );
             connect( job, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
             m_transferjob->addSubjob( job );
+
+            if( moodJob )
+            {
+                connect( moodJob, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
+                m_transferjob->addSubjob( moodJob );
+            }
+
             QString name = track->prettyName();
             if( track->artist() )
                 name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
@@ -605,6 +641,7 @@ bool SqlCollectionLocation::startNextRemoveJob()
         Meta::TrackPtr track = m_removetracks.takeFirst();
         // KUrl src = track->playableUrl();
         KUrl src = track->playableUrl();
+        KUrl srcMoodFile = moodFile( src );
 
         debug() << "isGoingToRemoveSources() " << isGoingToRemoveSources();
         if( isGoingToRemoveSources() && destination() ) // is organize operation?
@@ -623,6 +660,9 @@ bool SqlCollectionLocation::startNextRemoveJob()
         KIO::DeleteJob *job = KIO::del( src, KIO::HideProgressInfo );
         if( job )   //just to be safe
         {
+            if( QFile::exists( srcMoodFile.toLocalFile() ) )
+                KIO::del( srcMoodFile, KIO::HideProgressInfo );
+           
             connect( job, SIGNAL( result( KJob* ) ), SLOT( slotRemoveJobFinished( KJob* ) ) );
             QString name = track->prettyName();
             if( track->artist() )
@@ -637,6 +677,13 @@ bool SqlCollectionLocation::startNextRemoveJob()
     return false;
 }
 
+KUrl 
+SqlCollectionLocation::moodFile( const KUrl &track ) const
+{
+    KUrl moodPath = track;
+    moodPath.setFileName( "." + moodPath.fileName().replace( QRegExp( "(\\.\\w{2,5})$" ), ".mood" ) );
+    return moodPath;
+}
 
 TransferJob::TransferJob( SqlCollectionLocation * location, const Transcoding::Configuration & configuration )
     : KCompositeJob( 0 )
