@@ -307,22 +307,6 @@ MemoryQueryMaker::orderByRandom()
     return this;
 }
 
-QueryMaker*
-MemoryQueryMaker::includeCollection( const QString &collectionId )
-{
-    Q_UNUSED( collectionId );
-    //TODO stub
-    return this;
-}
-
-QueryMaker*
-MemoryQueryMaker::excludeCollection( const QString &collectionId )
-{
-    Q_UNUSED( collectionId );
-    //TODO stub
-    return this;
-}
-
 QueryMaker*
 MemoryQueryMaker::addMatch( const Meta::TrackPtr &track )
 {
