@@ -478,6 +478,7 @@ SqlPodcastChannel::fromPlaylistPtr( Playlists::PlaylistPtr playlist )
 SqlPodcastChannel::SqlPodcastChannel( SqlPodcastProvider *provider,
                                             const QStringList &result )
     : Podcasts::PodcastChannel()
+    , m_episodesLoaded( false )
     , m_provider( provider )
 {
     SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
@@ -497,7 +498,6 @@ SqlPodcastChannel::SqlPodcastChannel( SqlPodcastProvider *provider,
     m_purge = sqlStorage->boolTrue() == *(iter++);
     m_purgeCount = (*(iter++)).toInt();
     m_writeTags = sqlStorage->boolTrue() == *(iter++);
-    loadEpisodes();
 }
 
 SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
@@ -513,7 +513,7 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     m_subtitle = channel->subtitle();
     m_summary = channel->summary();
     m_author = channel->author();
-    
+
     // PodcastChannel
     m_url = channel->url();
     m_webLink = channel->webLink();
@@ -521,9 +521,9 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     m_labels = channel->labels();
     m_subscribeDate = channel->subscribeDate();
     m_copyright = channel->copyright();
-    
+
     if( channel->hasImage() )
-        m_image = channel->image().toImage();
+        m_image = channel->image();
 
     //Default Settings
 
@@ -544,6 +544,35 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
 
         m_episodes << SqlPodcastEpisodePtr( sqlEpisode );
     }
+    m_episodesLoaded = true;
+}
+
+int
+SqlPodcastChannel::trackCount() const
+{
+    if( m_episodesLoaded )
+        return m_episodes.count();
+
+    QString query = "SELECT COUNT(id) FROM podcastepisodes WHERE channel = %1";
+
+    SqlStorage *sql = CollectionManager::instance()->sqlStorage();
+    Q_ASSERT( sql );
+
+    QStringList results = sql->query( query.arg( m_dbId ) );
+    if( results.isEmpty() )
+    {
+        error() << "no results for COUNT query on playlist_tracks table!";
+        return -1;
+    }
+    int trackCount = results.first().toInt();
+    return m_purge ? qMin( m_purgeCount, trackCount ): trackCount;
+}
+
+void
+SqlPodcastChannel::triggerTrackLoad()
+{
+    if( !m_episodesLoaded )
+        loadEpisodes();
 }
 
 Playlists::PlaylistProvider *
@@ -618,23 +647,32 @@ SqlPodcastChannel::setImageUrl( const KUrl &imageUrl )
 Podcasts::PodcastEpisodePtr
 SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
 {
-    DEBUG_BLOCK
-    debug() << "adding episode " << episode->title() << " to sqlchannel " << title();
-
     if( !m_provider )
         return PodcastEpisodePtr();
 
-    //check for guid and return the episode we might already have.
-    if( !episode->guid().isEmpty() && m_provider->possiblyContainsTrack( episode->guid() ) )
+    KUrl checkUrl;
+    //searched in the database for guid or enclosure url
+    if( !episode->guid().isEmpty() )
+        checkUrl = episode->guid();
+    else if( !episode->uidUrl().isEmpty() )
+        checkUrl = episode->uidUrl();
+    else
+        return PodcastEpisodePtr(); //noting to check for
+
+    if( m_provider->possiblyContainsTrack( checkUrl ) )
         return PodcastEpisodePtr::dynamicCast( m_provider->trackForUrl( episode->guid() ) );
 
+    //force episodes load.
+    if( !m_episodesLoaded )
+        loadEpisodes();
+
     SqlPodcastEpisodePtr sqlEpisode = SqlPodcastEpisodePtr( new SqlPodcastEpisode( episode ) );
 
     //episodes are sorted on pubDate high to low
-    SqlPodcastEpisodeList::iterator i;
-    for( i = m_episodes.begin() ; i != m_episodes.end() ; ++i )
+    int i;
+    for( i = 0; i < m_episodes.count() ; i++ )
     {
-        if( sqlEpisode->pubDate() > (*i)->pubDate() )
+        if( sqlEpisode->pubDate() > m_episodes[i]->pubDate() )
         {
             m_episodes.insert( i, sqlEpisode );
             break;
@@ -642,19 +680,31 @@ SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
     }
 
     //insert in case the list is empty or at the end of the list
-    if( i == m_episodes.end() )
+    if( i == m_episodes.count() )
         m_episodes << sqlEpisode;
 
-    if( hasPurge() && m_episodes.count() > purgeCount() )
+    notifyObserversTrackAdded( Meta::TrackPtr::dynamicCast( sqlEpisode ), i );
+
+    applyPurge();
+
+    return PodcastEpisodePtr::dynamicCast( sqlEpisode );
+}
+
+void
+SqlPodcastChannel::applyPurge()
+{
+    if( !hasPurge() )
+        return;
+
+    while( m_episodes.count() > purgeCount() )
     {
-        debug() << "removing last episode from the list since we are limited to " << purgeCount();
         SqlPodcastEpisodePtr removedEpisode = m_episodes.takeLast();
         m_provider->deleteDownloadedEpisode( removedEpisode );
 
-        notifyObserversTrackRemoved( purgeCount() );
+        notifyObserversTrackRemoved( m_episodes.count() );
     }
 
-    return PodcastEpisodePtr::dynamicCast( sqlEpisode );
+    //TODO: load missing episodes in case m_episodes.count() <= purgeCount()
 }
 
 void
@@ -742,8 +792,11 @@ SqlPodcastChannel::loadEpisodes()
     for(int i=0; i < results.size(); i+=rowLength)
     {
         QStringList episodesResult = results.mid( i, rowLength );
-        SqlPodcastEpisode *episode = new SqlPodcastEpisode( episodesResult, SqlPodcastChannelPtr( this ) );
-        m_episodes << SqlPodcastEpisodePtr( episode );
+        SqlPodcastEpisodePtr sqlEpisode = SqlPodcastEpisodePtr(
+                new SqlPodcastEpisode( episodesResult, SqlPodcastChannelPtr( this ) ) );
+        m_episodes <<  sqlEpisode;
     }
+
+    m_episodesLoaded = true;
 }
 
