@@ -1,5 +1,7 @@
 /****************************************************************************************
  * Copyright (c) 2006 Giovanni Venturi <giovanni@kde-it.org>                            *
+ * Copyright (c) 2010 Ralf Engels <ralf-engels@gmx.de>                                  *
+ * Copyright (c) 2010 Sergey Ivanov <123kash@gmail.com>                                 *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -21,12 +23,9 @@
 #include "ui_EditFilterDialog.h"
 
 #include <KDialog>
-
 #include <QList>
 
-class QWidget;
-class QSpinBox;
-class QDateEdit;
+class TokenDropTarget;
 
 class EditFilterDialog : public KDialog
 {
@@ -42,49 +41,32 @@ class EditFilterDialog : public KDialog
         void filterChanged( const QString &filter );
 
     private:
+        void initTokenPool();
+        Token *tokenForField( const qint64 field );
+        void parseTextFilter( const QString &text );
+        void updateMetaQueryWidgetView();
+
+        struct Filter
+        {
+            MetaQueryWidget::Filter filter;
+            bool inverted;
+        };
+
         Ui::EditFilterDialog m_ui;
+        TokenDropTarget *m_dropTarget;
+        Token *m_curToken;
+        QMap< Token *, Filter > m_filters;
 
-        bool m_appended;               // true if a filter appended
-        QString m_filterText;          // the resulting filter string
-        QString m_previousFilterText;  // the previous resulting filter string
-
-        // Cache lists for completion
-        QStringList m_artists;
-        QStringList m_albums;
-        QStringList m_composers;
-        QStringList m_genres;
-        QStringList m_labels;
-
-        QString keywordConditionDate(const QString& keyword) const;
-        QString keywordConditionNumeric(const QString& keyword) const;
-        QString keywordConditionText(const QString& keyword) const;
+        QString m_separator;
 
     private slots:
-        void selectedAttribute( const QString &attr );
-
-        void minSpinChanged(int value);
-        void maxSpinChanged(int value);
-
-        void dateWanted();
-        void textWanted( const KIcon &icon = KIcon() );
-        void textWanted( const QStringList &completions, const KIcon &icon = KIcon() );
-        void valueWanted();
-
-        void chooseCondition(int index);
-        void chooseOneValue();
-        void chooseMinMaxValue();
-        
-        void resultReady( const QString &collectionId, const Meta::AlbumList &albums );
-        void resultReady( const QString &collectionId, const Meta::ArtistList &artists );
-        void resultReady( const QString &collectionId, const Meta::ComposerList &composers );
-        void resultReady( const QString &collectionId, const Meta::GenreList &genres );
-        void resultReady( const QString &collectionId, const Meta::LabelList &labels );
-
-    protected slots:
-        virtual void slotDefault();
-        virtual void slotUser1();
-        virtual void slotUser2();
-        virtual void slotOk();
+        void slotTokenSelected( QWidget *token );
+        void slotTokenDropTargetChanged();
+        void slotAttributeChanged( const MetaQueryWidget::Filter &filter );
+        void slotInvert( bool checked );
+        void slotSeparatorChange( const QString &separator );
+        void slotReset();
+        void accept();
 };
 
 #endif /* AMAROK_EDITFILTERDIALOG_H */
