@@ -21,9 +21,10 @@
 #include "TrackOrganizer.h"
 
 #include "core/support/Amarok.h"
-#include "core/support/Debug.h"
 #include "QStringx.h"
 
+#include <KLocale>
+
 TrackOrganizer::TrackOrganizer( const Meta::TrackList &tracks, QObject* parent )
     : QObject( parent )
     , m_allTracks( tracks )
@@ -84,7 +85,10 @@ QString TrackOrganizer::buildDestination(const QString& format, const Meta::Trac
     args["artist"] = artist;
     args["albumartist"] = albumartist;
     args["initial"] = albumartist.mid( 0, 1 ).toUpper();    //artists starting with The are already handled above
-    args["filetype"] = track->type();
+    if( m_targetFileExtension == QString() )
+        args["filetype"] = track->type();
+    else
+        args["filetype"] = m_targetFileExtension;
     args["rating"] = track->rating();
     args["filesize"] = track->filesize();
     args["length"] = track->length() / 1000;
@@ -100,7 +104,8 @@ QString TrackOrganizer::buildDestination(const QString& format, const Meta::Trac
     if( !result.startsWith( '/' ) )
         result.prepend( "/" );
 
-   return result.replace( QRegExp( "/\\.*" ), "/" );
+    QFileInfo path( result );           // Used to polish path string. (e.g. remove '//')
+    return path.absoluteFilePath();
 }
 
 QString TrackOrganizer::cleanPath( const QString& component ) const
@@ -174,3 +179,8 @@ void TrackOrganizer::setReplace(const QString& regex, const QString& string)
     m_regexPattern = regex;
     m_replaceString = string;
 }
+
+void TrackOrganizer::setTargetFileExtension( const QString &fileExtension )
+{
+    m_targetFileExtension = fileExtension;
+}
