@@ -30,8 +30,6 @@
 #include "core/meta/support/MetaConstants.h"
 #include "core/collections/MetaQueryMaker.h"
 #include "core/collections/QueryMaker.h"
-#include "core-impl/collections/support/XmlQueryReader.h"
-#include "core-impl/collections/support/XmlQueryWriter.h"
 
 #include <QMutexLocker>
 #include <QDateTime>
@@ -61,16 +59,6 @@ Dynamic::Bias::fromXml( QDomElement e )
     if( type == "global" )
     {
         return Dynamic::GlobalBias::fromXml( e );
-#if QT_VERSION >= 0x040600
-#else
-                    // QXmlStreamReader::readNextStartElement doesn't exist in Qt-4.5
-                    // this is the inlined method body, adapted from lines 656 - 665 of
-                    // http://qt.gitorious.org/qt/qt/blobs/4.6/src/corelib/xml/qxmlstream.cpp
-                    while ( reader.readNext() != QXmlStreamReader::Invalid ) {
-                        if ( reader.isEndElement() || reader.isStartElement() )
-                            break;
-                    }
-#endif
     }
     else if( type == "custom" )
     {
@@ -181,16 +169,14 @@ Dynamic::CollectionDependantBias::collectionUpdated()
     m_needsUpdating = true;
 }
 
-Dynamic::GlobalBias::GlobalBias( double weight, Filter filter )
-    : m_qm(0)
+Dynamic::GlobalBias::GlobalBias( double weight, MetaQueryWidget::Filter filter )
 {
     setWeight( weight );
     setFilter( filter );
 }
 
-Dynamic::GlobalBias::GlobalBias( Collections::Collection* coll, double weight, Filter filter )
+Dynamic::GlobalBias::GlobalBias( Collections::Collection* coll, double weight, MetaQueryWidget::Filter filter )
     : CollectionDependantBias( coll )
-    , m_qm(0)
 {
     setWeight( weight );
     setFilter( filter );
@@ -198,26 +184,28 @@ Dynamic::GlobalBias::GlobalBias( Collections::Collection* coll, double weight, F
 
 Dynamic::GlobalBias::~GlobalBias()
 {
-    delete m_qm;
+    delete m_qm.data();
 }
 
 QString
-Dynamic::GlobalBias::filterConditionToString( Dynamic::GlobalBias::FilterCondition cond )
+Dynamic::GlobalBias::filterConditionToString( MetaQueryWidget::FilterCondition cond )
 {
     switch( cond )
     {
-        case Equals:
-            return "equals";
-        case GreaterThan:
-            return "greater";
-        case LessThan:
-            return "less";
-        case Between:
-            return "between";
-        case OlderThan:
-            return "older";
-        case Contains:
-            return "contains";
+    case MetaQueryWidget::Equals:
+        return "equals";
+    case MetaQueryWidget::GreaterThan:
+        return "greater";
+    case MetaQueryWidget::LessThan:
+        return "less";
+    case MetaQueryWidget::Between:
+        return "between";
+    case MetaQueryWidget::OlderThan:
+        return "older";
+    case MetaQueryWidget::Contains:
+        return "contains";
+    default:
+        ;// the other conditions are only for the advanced playlist generator
     }
     return QString();
 }
@@ -226,7 +214,7 @@ Dynamic::GlobalBias*
 Dynamic::GlobalBias::fromXml( QDomElement e )
 {
     double weight = 0.0;
-    Dynamic::GlobalBias::Filter filter;
+    MetaQueryWidget::Filter filter;
 
     QDomElement queryElement = e.firstChildElement( "query" );
     if( !queryElement.isNull() )
@@ -238,17 +226,17 @@ Dynamic::GlobalBias::fromXml( QDomElement e )
             if( !includeElement.isNull() )
             {
                 QString field = includeElement.attribute("field");
-                filter.field = XmlQueryReader::fieldVal( QStringRef(&field) );
+                filter.field = Meta::fieldForName( field );
                 filter.value = includeElement.attribute( "value", "" );
                 filter.numValue = filter.value.toLongLong();
                 filter.numValue2 = includeElement.attribute( "value2", 0 ).toLongLong();
 
                 QString condition = includeElement.attribute( "compare", "" );
-                filter.condition = Contains;
+                filter.condition = MetaQueryWidget::Contains;
                 for( int i=0; i<5; i++ )
                 {
-                    if( condition == filterConditionToString( (FilterCondition)i ) )
-                        filter.condition = (FilterCondition)i;
+                    if( condition == filterConditionToString( (MetaQueryWidget::FilterCondition)i ) )
+                        filter.condition = (MetaQueryWidget::FilterCondition)i;
                 }
             }
         }
@@ -279,8 +267,8 @@ Dynamic::GlobalBias::xml() const
     QDomElement queryElement = doc.createElement( "query" );
     QDomElement filtersElement = doc.createElement( "filters" );
     QDomElement includeElement = doc.createElement( "include" );
-    includeElement.setAttribute( "field", Collections::XmlQueryWriter::fieldName( m_filter.field ) );
-    if( m_filter.condition == Contains )
+    includeElement.setAttribute( "field", Meta::nameForField( m_filter.field ) );
+    if( m_filter.condition == MetaQueryWidget::Contains )
         includeElement.setAttribute( "value", m_filter.value );
     else
         includeElement.setAttribute( "value", QString::number( m_filter.numValue ) );
@@ -331,59 +319,84 @@ Dynamic::CollectionFilterCapability* Dynamic::GlobalBias::collectionFilterCapabi
 
 
 
-Dynamic::GlobalBias::Filter
+MetaQueryWidget::Filter
 Dynamic::GlobalBias::filter() const
 {
     return m_filter;
 }
 
 void
-Dynamic::GlobalBias::setFilter( const Filter &filter)
+Dynamic::GlobalBias::setFilter( const MetaQueryWidget::Filter &filter)
 {
     DEBUG_BLOCK
     QMutexLocker locker( &m_mutex );
 
+    if (m_qm)
+        delete m_qm.data();
+
+    // the whole bias/collection management is not very robust.
+    // here we do our best to get a collection and a query maker
+    // when in principle a bias should be collection independent.
     if( !m_collection )
         m_collection = CollectionManager::instance()->primaryCollection();
+    if( !m_collection )
+        m_qm = CollectionManager::instance()->queryMaker();
+    else
+        m_qm = m_collection->queryMaker();
 
-    if (m_qm)
-        delete m_qm;
-
-    m_qm = m_collection->queryMaker();
-
-    if( filter.field != 0 )
+    switch( filter.condition )
     {
-        switch( filter.condition )
-        {
-        case Equals:
-        case GreaterThan:
-        case LessThan:
-            m_qm->addNumberFilter( filter.field, filter.numValue,
-                    (Collections::QueryMaker::NumberComparison)filter.condition );
-            break;
-        case Between:
-            m_qm->addNumberFilter( filter.field, qMin(filter.numValue, filter.numValue2)-1,
-                    Collections::QueryMaker::GreaterThan );
-            m_qm->addNumberFilter( filter.field, qMax(filter.numValue, filter.numValue2)+1,
-                    Collections::QueryMaker::LessThan );
-            break;
-        case OlderThan:
-            m_qm->addNumberFilter( filter.field, QDateTime::currentDateTime().toTime_t() - filter.numValue,
-                    Collections::QueryMaker::LessThan );
-            break;
+    case MetaQueryWidget::Equals:
+    case MetaQueryWidget::GreaterThan:
+    case MetaQueryWidget::LessThan:
+        m_qm.data()->addNumberFilter( filter.field, filter.numValue,
+                               (Collections::QueryMaker::NumberComparison)filter.condition );
+        break;
+    case MetaQueryWidget::Between:
+        m_qm.data()->beginAnd();
+        m_qm.data()->addNumberFilter( filter.field, qMin(filter.numValue, filter.numValue2)-1,
+                               Collections::QueryMaker::GreaterThan );
+        m_qm.data()->addNumberFilter( filter.field, qMax(filter.numValue, filter.numValue2)+1,
+                               Collections::QueryMaker::LessThan );
+        m_qm.data()->endAndOr();
+        break;
+    case MetaQueryWidget::OlderThan:
+        m_qm.data()->addNumberFilter( filter.field, QDateTime::currentDateTime().toTime_t() - filter.numValue,
+                               Collections::QueryMaker::LessThan );
+        break;
 
-        case Contains:
-            m_qm->addFilter( filter.field, filter.value );
+    case MetaQueryWidget::Contains:
+        if( filter.field == 0 )
+        {
+            // simple search
+            // TODO: split different words and make seperate searches
+            m_qm.data()->beginOr();
+            m_qm.data()->addFilter( Meta::valArtist,  filter.value );
+            m_qm.data()->addFilter( Meta::valTitle,   filter.value );
+            m_qm.data()->addFilter( Meta::valAlbum,   filter.value );
+            m_qm.data()->addFilter( Meta::valGenre,   filter.value );
+            m_qm.data()->addFilter( Meta::valUrl,     filter.value );
+            m_qm.data()->addFilter( Meta::valComment, filter.value );
+            m_qm.data()->addFilter( Meta::valLabel,   filter.value );
+            m_qm.data()->endAndOr();
         }
+        else
+        {
+            m_qm.data()->addFilter( filter.field, filter.value );
+        }
+        break;
+
+    default:
+        ;// the other conditions are only for the advanced playlist generator
     }
 
-    m_qm->setQueryType( Collections::QueryMaker::Custom );
-    m_qm->addReturnValue( Meta::valUniqueId );
-    m_qm->orderByRandom(); // as to not affect the amortized time
+    m_qm.data()->setQueryType( Collections::QueryMaker::Custom );
+    m_qm.data()->addReturnValue( Meta::valUniqueId );
+    m_qm.data()->orderByRandom(); // as to not affect the amortized time
 
-    connect( m_qm, SIGNAL(newResultReady( QString, QStringList )),
+    connect( m_qm.data(), SIGNAL(newResultReady( QString, QStringList )),
             SLOT(updateReady( QString, QStringList )), Qt::DirectConnection );
-    connect( m_qm, SIGNAL(queryDone()), SLOT(updateFinished()), Qt::DirectConnection );
+    connect( m_qm.data(), SIGNAL(queryDone()), SLOT(updateFinished()), Qt::DirectConnection );
 
     m_filter = filter;
     locker.unlock(); // because collectionUpdate also want's a lock
@@ -447,7 +460,7 @@ void Dynamic::GlobalBias::update()
     if( !m_needsUpdating )
         return;
 
-    m_qm->run();
+    m_qm.data()->run();
 }
 
 void
@@ -468,8 +481,7 @@ Dynamic::GlobalBias::updateReady( QString collectionId, QStringList uids )
 
     QMutexLocker locker( &m_mutex );
 
-    int protocolLength =
-        (QString(m_collection->uidUrlProtocol()) + "://").length();
+    const int protocolLength = QString( m_collection->uidUrlProtocol() + "://" ).length();
 
     m_property.clear();
     m_property.reserve( uids.size() );
@@ -583,11 +595,10 @@ Dynamic::NormalBias::energy( const Meta::TrackList& playlist, const Meta::TrackL
 {
     Q_UNUSED(context)
 
-
     QList<double> fields;
 
     foreach( Meta::TrackPtr t, playlist )
-        fields += relevantField(t) - m_mu;
+        fields += Meta::valueForField( m_field, t ).toDouble() - m_mu;
 
     qSort( fields );
 
@@ -609,33 +620,6 @@ Dynamic::NormalBias::energy( const Meta::TrackList& playlist, const Meta::TrackL
     return D;
 }
 
-double
-Dynamic::NormalBias::relevantField( Meta::TrackPtr track ) const
-{
-    if( !track )
-        return m_mu;
-    if( m_field == Meta::valYear && track->year() )
-        return (double)track->year()->name().toInt();
-    if( m_field == Meta::valPlaycount )
-        return (double)track->playCount();
-    if( m_field == Meta::valRating )
-        return (double)track->rating();
-    if( m_field == Meta::valScore )
-        return track->score();
-    if( m_field == Meta::valLength )
-        return (double)track->length();
-    if( m_field == Meta::valTrackNr )
-        return (double)track->trackNumber();
-    if( m_field == Meta::valDiscNr )
-        return (double)track->discNumber();
-    if( m_field == Meta::valFirstPlayed )
-        return (double)track->firstPlayed();
-    if( m_field == Meta::valLastPlayed )
-        return (double)track->lastPlayed();
-
-    return m_mu;
-}
-
 double
 Dynamic::NormalBias::sigmaFromScale( double scale )
 {
@@ -655,26 +639,6 @@ Dynamic::NormalBias::sigmaFromScale( double scale )
         minStdDev = 0.5;
         maxStdDev = 10.0;
     }
-    else if( m_field == Meta::valPlaycount )
-    {
-        minStdDev = 0.5;
-        maxStdDev = 50.0;
-    }
-    else if( m_field == Meta::valRating )
-    {
-        minStdDev = 0.5;
-        maxStdDev = 2.5;
-    }
-    else if( m_field == Meta::valScore )
-    {
-        minStdDev = 1.0;
-        maxStdDev = 50.0;
-    }
-    else if( m_field == Meta::valLength )
-    {
-        minStdDev = 10.0;
-        maxStdDev = 240.0;
-    }
     else if( m_field == Meta::valTrackNr )
     {
         minStdDev = 0.5;
@@ -685,15 +649,57 @@ Dynamic::NormalBias::sigmaFromScale( double scale )
         minStdDev = 0.5;
         maxStdDev = 5.0;
     }
-    else if( m_field == Meta::valFirstPlayed )
+    else if( m_field == Meta::valBpm )
+    {
+        minStdDev = 60;
+        maxStdDev = 200.0;
+    }
+    else if( m_field == Meta::valLength )
+    {
+        minStdDev = 60;
+        maxStdDev = 60.0 * 60.0 * 2.0; // two hours
+    }
+    else if( m_field == Meta::valBitrate )
+    {
+        minStdDev = 60;
+        maxStdDev = 400.0;
+    }
+    else if( m_field == Meta::valSamplerate )
+    {
+        minStdDev = 8000;
+        maxStdDev = 48000.0;
+    }
+    else if( m_field == Meta::valFilesize )
+    {
+        minStdDev = 0;
+        maxStdDev = 200000.0;
+    }
+    else if( m_field == Meta::valScore )
+    {
+        minStdDev = 1.0;
+        maxStdDev = 50.0;
+    }
+    else if( m_field == Meta::valRating )
+    {
+        minStdDev = 0.5;
+        maxStdDev = 2.5;
+    }
+    else if( m_field == Meta::valLength )
+    {
+        minStdDev = 10.0;
+        maxStdDev = 240.0;
+    }
+    else if( m_field == Meta::valCreateDate ||
+             m_field == Meta::valFirstPlayed ||
+             m_field == Meta::valLastPlayed )
     {
         minStdDev = 3600.0;   // one hour
         maxStdDev = 604800.0; // one week
     }
-    else if( m_field == Meta::valLastPlayed )
+    else if( m_field == Meta::valPlaycount )
     {
-        minStdDev = 3600.0;   // one hour
-        maxStdDev = 604800.0; // one week
+        minStdDev = 0.5;
+        maxStdDev = 50.0;
     }
 
     // linear interpolation between min and max std. dev.
