@@ -10,7 +10,7 @@
  *                                                                                      *
  * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
- * PARTICULAR PURPOSE. See the GNU General Public License for more details.              *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
  *                                                                                      *
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
@@ -19,7 +19,7 @@
 #ifndef AMAROK_DYNAMICPLAYLIST_H
 #define AMAROK_DYNAMICPLAYLIST_H
 
-#include "Meta.h"
+#include "core/meta/Meta.h"
 
 #include <QDomElement>
 #include <QSharedData>
@@ -36,7 +36,7 @@ class DynamicPlaylist : public QObject, public QSharedData
     Q_OBJECT
 
     public:
-        DynamicPlaylist( Amarok::Collection* coll = 0 );
+        DynamicPlaylist( Collections::Collection* coll = 0 );
 
         virtual ~DynamicPlaylist();
 
@@ -47,7 +47,8 @@ class DynamicPlaylist : public QObject, public QSharedData
         QString title() const;
 
         void setTitle( QString );
-
+        virtual void setActive(bool active);
+        
         virtual void requestAbort() {}
 
     signals:
@@ -55,10 +56,12 @@ class DynamicPlaylist : public QObject, public QSharedData
 
     public slots:
         virtual void recalculate();
-
+        virtual void invalidate();
+        
     protected:
-        Amarok::Collection* m_collection;
+        Collections::Collection* m_collection;
         QString m_title;
+        bool m_active;
 };
 
 
