@@ -66,14 +66,16 @@ ConstraintTypes::PlaylistDuration::PlaylistDuration( QDomElement& xmlelem, Const
     DEBUG_BLOCK
     QDomAttr a;
 
-    // I the duration seems to have been a PlaylistLength before.
-    // the attribute used there was called "length".
-    // So we parse it, just to be on the safe side.
     a = xmlelem.attributeNode( "duration" );
-    if ( a.isNull() )
+    if ( !a.isNull() ) {
+        m_duration = a.value().toInt();
+    } else {
+        // Accomodate schema change when PlaylistLength became PlaylistDuration
         a = xmlelem.attributeNode( "length" );
-    if ( !a.isNull() )
-        m_duration = a.value().toLongLong();
+        if ( !a.isNull() )
+            m_duration = a.value().toInt();
+    }
+
 
     a = xmlelem.attributeNode( "comparison" );
     if ( !a.isNull() )
@@ -101,7 +103,7 @@ ConstraintTypes::PlaylistDuration::editWidget() const
 {
     PlaylistDurationEditWidget* e = new PlaylistDurationEditWidget( m_duration, m_comparison, static_cast<int>( 10*m_strictness ) );
     connect( e, SIGNAL( comparisonChanged( const int ) ), this, SLOT( setComparison( const int ) ) );
-    connect( e, SIGNAL( durationChanged( const qint64 ) ), this, SLOT( setDuration( const qint64 ) ) );
+    connect( e, SIGNAL( durationChanged( const int ) ), this, SLOT( setDuration( const int ) ) );
     connect( e, SIGNAL( strictnessChanged( const int ) ), this, SLOT( setStrictness( const int ) ) );
     return e;
 }
@@ -294,7 +296,7 @@ ConstraintTypes::PlaylistDuration::setComparison( const int c )
 }
 
 void
-ConstraintTypes::PlaylistDuration::setDuration( const qint64 v )
+ConstraintTypes::PlaylistDuration::setDuration( const int v )
 {
     m_duration = v;
     emit dataChanged();
@@ -310,7 +312,7 @@ ConstraintTypes::PlaylistDuration::setStrictness( const int sv )
  * Edit Widget                *
  ******************************/
 
-ConstraintTypes::PlaylistDurationEditWidget::PlaylistDurationEditWidget( const qint64 duration,
+ConstraintTypes::PlaylistDurationEditWidget::PlaylistDurationEditWidget( const int duration,
                                                                      const int comparison,
                                                                      const int strictness ) : QWidget( 0 )
 {
