@@ -112,6 +112,9 @@ PlaylistManager::addProvider( Playlists::PlaylistProvider *provider, int categor
     if( !m_providerMap.uniqueKeys().contains( category ) )
             newCategory = true;
 
+    //disconnect all signals connected to this object to be sure.
+    provider->disconnect( this, 0 );
+
     m_providerMap.insert( category, provider );
     connect( provider, SIGNAL(updated()), SLOT(slotUpdated()));
     connect( provider, SIGNAL(playlistAdded( Playlists::PlaylistPtr )),
@@ -120,10 +123,10 @@ PlaylistManager::addProvider( Playlists::PlaylistProvider *provider, int categor
              SLOT(slotPlaylistRemoved( Playlists::PlaylistPtr )));
 
     if( newCategory )
-        emit( categoryAdded( category ) );
+        emit categoryAdded( category );
 
-    emit( providerAdded( provider, category ) );
-    emit( updated() );
+    emit providerAdded( provider, category );
+    emit updated( category );
 
     loadPlaylists( provider, category );
 }
@@ -141,17 +144,22 @@ PlaylistManager::addPlaylist( Playlists::PlaylistPtr playlist, int category )
     if( shouldBeSynced( playlist ) )
     {
         SyncedPlaylistPtr syncedPlaylist = m_syncRelStore->asSyncedPlaylist( playlist );
+        Playlists::PlaylistPtr syncedPlaylistPtr =
+                Playlists::PlaylistPtr::dynamicCast( syncedPlaylist );
         m_syncedPlaylistMap.insert( syncedPlaylist, playlist );
         if( !m_playlistMap.values( category ).contains(
-                Playlists::PlaylistPtr::dynamicCast( syncedPlaylist ) ) )
+                Playlists::PlaylistPtr::dynamicCast( syncedPlaylistPtr ) ) )
         {
-            m_playlistMap.insert( category,
-                                  Playlists::PlaylistPtr::dynamicCast( syncedPlaylist ) );
+            m_playlistMap.insert( category, syncedPlaylistPtr );
+            //reemit so models know about new playlist in their category
+            emit playlistAdded( syncedPlaylistPtr, category );
         }
     }
     else
     {
         m_playlistMap.insert( category, playlist );
+        //reemit so models know about new playlist in their category
+        emit playlistAdded( playlist, category );
     }
 }
 
@@ -172,8 +180,8 @@ PlaylistManager::removeProvider( Playlists::PlaylistProvider *provider )
 
     m_providerMap.remove( provider->category(), provider );
 
-    emit( providerRemoved( provider, provider->category() ) );
-    emit( updated() );
+    emit providerRemoved( provider, provider->category() );
+    emit updated( provider->category() );
 }
 
 void
@@ -198,6 +206,8 @@ PlaylistManager::removePlaylist( Playlists::PlaylistPtr playlist, int category )
     {
         m_playlistMap.remove( category, playlist );
     }
+
+    emit playlistRemoved( playlist, category );
 }
 
 void
@@ -212,21 +222,19 @@ PlaylistManager::slotUpdated()
     //This is an expensive operation, the provider should use playlistAdded/Removed signals instead.
     removePlaylists( provider );
     loadPlaylists( provider, provider->category() );
-    emit( updated() );
+    emit updated( provider->category() );
 }
 
 void
 PlaylistManager::slotPlaylistAdded( Playlists::PlaylistPtr playlist )
 {
     addPlaylist( playlist, playlist->provider()->category() );
-    emit updated();
 }
 
 void
 PlaylistManager::slotPlaylistRemoved( Playlists::PlaylistPtr playlist )
 {
     removePlaylist( playlist, playlist->provider()->category() );
-    emit updated();
 }
 
 Playlists::PlaylistList
@@ -292,28 +300,16 @@ bool
 PlaylistManager::save( Meta::TrackList tracks, const QString &name,
                        Playlists::UserPlaylistProvider *toProvider )
 {
-    AMAROK_DEPRECATED
-    // used by: Playlist::Widget::slotSaveCurrentPlaylist()
     //if toProvider is 0 use the default Playlists::UserPlaylistProvider (SQL)
     Playlists::UserPlaylistProvider *prov = toProvider ? toProvider : m_defaultUserPlaylistProvider;
-    Playlists::PlaylistPtr playlist = Playlists::PlaylistPtr();
-    if( name.isEmpty() )
-    {
-        debug() << "Empty name of playlist, or editing now";
-        playlist = prov->save( tracks );
-        if( playlist.isNull() )
-            return false;
+    Playlists::PlaylistPtr playlist = prov->save( tracks, name );
+    if( playlist.isNull() )
+        return false;
 
-        AmarokUrl("amarok://navigate/playlists/user playlists").run();
-        emit( renamePlaylist( playlist ) );
-    }
-    else
-    {
-        debug() << "Playlist is being saved with name: " << name;
-        playlist = prov->save( tracks, name );
-    }
+    AmarokUrl("amarok://navigate/playlists/user playlists").run();
+    emit renamePlaylist( playlist );
 
-    return !playlist.isNull();
+    return true;
 }
 
 bool
@@ -350,7 +346,6 @@ PlaylistManager::rename( Playlists::PlaylistPtr playlist )
     {
         debug() << "Changing name from " << playlist->name() << " to " << newName.trimmed();
         provider->rename( playlist, newName.trimmed() );
-        emit( updated() );
     }
 }
 
