@@ -23,7 +23,7 @@
 #include "meta/LastFmMeta.h"
 #include "ServiceMetaBase.h"
 
-#include "collection/support/MemoryQueryMaker.h"
+#include "core-impl/collections/support/MemoryQueryMaker.h"
 
 #include <lastfm/ws.h>
 #include <lastfm/XmlQuery>
@@ -32,6 +32,8 @@
 
 #include <KLocale>
 
+using namespace Collections;
+
 LastFmServiceCollection::LastFmServiceCollection( const QString& userName )
     : ServiceCollection( 0, "last.fm", "last.fm" )
 {
