@@ -9,7 +9,7 @@
  *                                                                                      *
  * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
- * PARTICULAR PURPOSE. See the GNU General Public License for more details.              *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
  *                                                                                      *
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
@@ -27,6 +27,8 @@ namespace Meta
     class ServiceGenre;
 }
 
+namespace Collections {
+
 class LastFmServiceCollection : public ServiceCollection
 {
     Q_OBJECT
@@ -59,4 +61,6 @@ private:
     Meta::ServiceGenre *m_recentlyPlayed;
 };
 
+} //namespace Collections
+
 #endif // LASTFMSERVICECOLLECTION_H
