@@ -50,12 +50,20 @@
 
 #include <typeinfo>
 
-AMAROK_EXPORT_PLUGIN( MagnatuneServiceFactory )
+AMAROK_EXPORT_SERVICE_PLUGIN( magnatunestore, MagnatuneServiceFactory )
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // class MagnatuneServiceFactory
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+MagnatuneServiceFactory::MagnatuneServiceFactory( QObject *parent, const QVariantList &args )
+    : ServiceFactory( parent, args )
+{
+    KPluginInfo pluginInfo( "amarok_service_magnatunestore.desktop", "services" );
+    pluginInfo.setConfig( config() );
+    m_info = pluginInfo;
+}
+
 void MagnatuneServiceFactory::init()
 {
     DEBUG_BLOCK
@@ -70,13 +78,6 @@ QString MagnatuneServiceFactory::name()
     return "Magnatune.com";
 }
 
-KPluginInfo MagnatuneServiceFactory::info()
-{
-    KPluginInfo pluginInfo( "amarok_service_magnatunestore.desktop", "services" );
-    pluginInfo.setConfig( config() );
-    return pluginInfo;
-}
-
 KConfigGroup MagnatuneServiceFactory::config()
 {
     return Amarok::config( "Service_Magnatune" );
