@@ -17,7 +17,7 @@
 
 #include "Mp3tunesMeta.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
 
 #include <KIcon>
 #include <KLocale>
