@@ -55,11 +55,10 @@ TimeDistanceWidget::TimeDistanceWidget( QWidget *parent )
 
     m_unitSelection = new KComboBox(this);
     connect( m_timeEdit, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateComboBoxLabels(int)) );
-    m_unitSelection->addItem( i18np( "second", "seconds", 0 ) );
-    m_unitSelection->addItem( i18np( "minute", "minutes", 0 ) );
-    m_unitSelection->addItem( i18np( "hour", "hours", 0 ) );
-    m_unitSelection->addItem( i18np( "day", "days", 0 ) );
-    m_unitSelection->addItem( i18np( "month", "months", 0 ) );
+    for (int i = 0; i < 5; ++i) {
+        m_unitSelection->addItem( QString() );
+    }
+    slotUpdateComboBoxLabels( 0 );
 
     QHBoxLayout *hLayout = new QHBoxLayout(this);
     hLayout->setContentsMargins(0, 0, 0, 0);
@@ -231,8 +230,8 @@ MetaQueryWidget::makeFieldSelection()
         addIconItem( m_fieldSelection, Meta::valUrl );
         // note: what about directory?
         addIconItem( m_fieldSelection, Meta::valTitle );
-        // note: album artist is not handled here. could be added
         addIconItem( m_fieldSelection, Meta::valArtist );
+        addIconItem( m_fieldSelection, Meta::valAlbumArtist );
         addIconItem( m_fieldSelection, Meta::valAlbum );
         addIconItem( m_fieldSelection, Meta::valGenre );
         addIconItem( m_fieldSelection, Meta::valComposer );
@@ -267,15 +266,8 @@ MetaQueryWidget::fieldChanged( int i )
     if( m_settingFilter )
         return;
 
-    qint64 field = Contains;
-    if( m_fieldSelection->count() == 0 )
-    {
-        if( isNumeric( field ) )
-            m_filter.condition = Equals;
-        else if( !isNumeric( field ) )
-            m_filter.condition = Contains;
-    }
-    else if( i<0 || i>=m_fieldSelection->count() )
+    qint64 field;
+    if( i<0 || i>=m_fieldSelection->count() )
         field = m_fieldSelection->itemData( 0 ).toInt();
     else
         field = m_fieldSelection->itemData( i ).toInt();
@@ -297,6 +289,9 @@ MetaQueryWidget::fieldChanged( int i )
         m_filter.numValue2 = 0;
     }
 
+    if( field && !m_apgMode )
+        m_filter.condition = Equals;
+
     m_filter.field = field;
 
     // in the fieldChanged slot we assume that the field was really changed,
@@ -535,6 +530,7 @@ MetaQueryWidget::makeValueSelection()
         // We,re not going to populate this. There tends to be too many titles.
         makeGenericComboSelection( true, 0 );
     else if( field == Meta::valArtist ||
+        field == Meta::valAlbumArtist ||
         field == Meta::valAlbum ||
         field == Meta::valGenre ||
         field == Meta::valComposer )
@@ -1062,5 +1058,25 @@ QString MetaQueryWidget::Filter::toString( bool invert )
     return result;
 }
 
+bool
+MetaQueryWidget::isFieldSelectorHidden() const
+{
+    return m_fieldSelection->isHidden();
+}
+
+void
+MetaQueryWidget::setFieldSelectorHidden( const bool hidden )
+{
+    m_fieldSelection->setVisible( !hidden );
+}
+
+void
+MetaQueryWidget::setField( const qint64 field )
+{
+    m_filter = Filter();
+    int index = m_fieldSelection->findData( field );
+    m_fieldSelection->setCurrentIndex( index == -1 ? 0 : index );
+}
+
 #include "MetaQueryWidget.moc"
 
